from django.conf import settings

if not settings.configured:
    settings.configure(
        DEBUG=True,
        INSTALLED_APPS=(
            'metaredirect',
        ),
        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': ':memory:',
            },
        },
        ROOT_URLCONF='metaredirect.tests.urls',
    )


from metaredirect.tests.tests import *  # NOQA


def run():
    import sys
    from django.test.utils import get_runner

    runner = get_runner(settings)()
    failures = runner.run_tests(('metaredirect',))
    sys.exit(failures)



from django import template


register = template.Library()


@register.filter
def escapeforwardslashes(value):
    """
    Escapes forward slashes in a string. For example, ``foo/bar`` becomes
    ``foo\/bar`` after the filter is applied.

    Useful for escaping slashes in URLs that will be used in an HTML <script>
    tag: http://stackoverflow.com/questions/1580647/

    This is possibly unnecessary when paired with the ``escapejs`` filter, but
    better safe then sorry.
    """
    return unicode(value).replace('/', '\/')



from django.http import HttpResponseRedirect, HttpResponsePermanentRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext

from metaredirect.helpers import is_interactive_user_agent


def redirect_to(request, url, permanent=False):
    """
    A generic view for redirecting to the provided URL, using a HTTP 200
    response with a META-tag based redirect for whitelisted interactive
    browsers, and falling back to a 300-class HTTP standard redirect for
    non-interactive browers.

    :param request: the original HTTP request triggering the response
    :type request: :class:`django.http.HttpRequest`
    :param url: the URL to redirect to
    :type url: :class:`str`
    :param permanent: if the redirect is permanent or not. Only used for
        non-interactive clients.
    :type permanent: :class:`bool`
    :returns: an HTTP response
    :rtype: :class:`django.http.HttpResponse` or derived class
    """
    if is_interactive_user_agent(request):
        context = RequestContext(request, {'url': url})
        response = render_to_response('metaredirect/redirect.txt',
            context_instance=context)
        response['X-Location'] = url
        return response
    else:
        if permanent:
            response_cls = HttpResponsePermanentRedirect
        else:
            response_cls = HttpResponseRedirect
        return response_cls(url)



import httpagentparser
import logging


logger = logging.getLogger(__name__)


def is_interactive_user_agent(request):
    """
    Returns if this request appears to have been made with an interactive user
    agent (e.g. a graphical browser).
    """
    user_agent = request.META.get('HTTP_USER_AGENT', '')
    properties = httpagentparser.detect(user_agent)
    if not properties:
        return False

    browser = properties.get('browser')

    # XXX: `httpagentparser.Result` overrides `dict.__missing__` to return an
    # empty string instead of raising a `KeyError`. In this case, we can't make
    # a reasonable judgement about what browser this is, so just assume it's
    # not an interactive one.
    if not browser:
        logger.info('Skipping unparsable user agent: %s', user_agent)
        return False

    return browser['name'] in ('Firefox', 'SeaMonkey', 'Konqueror', 'Opera',
        'Netscape', 'Microsoft Internet Explorer', 'Safari', 'Chrome')



#!/usr/bin/env python
import os
import sys

from setuptools import find_packages, setup


def get_version():
    from metaredirect import __version__
    return '.'.join(map(str, __version__))

try:
    version = get_version()
except ImportError:
    sys.path.append(os.path.join(os.path.dirname(__file__), 'metaredirect'))
    version = get_version()


install_requires = ['django', 'httpagentparser']
tests_require = ['mock']

setup(name='django-metaredirect',
    version=version,
    url='http://github.com/disqus/django-metaredirect/',
    author='ted kaemming',
    author_email='ted@disqus.com',
    description='META-tag and JavaScript based generic redirect views for '
        'maintaining HTTP referrers.',
    packages=find_packages(),
    include_package_data=True,
    install_requires=install_requires,
    license='Apache License 2.0',
    tests_require=tests_require,
    test_suite='metaredirect.tests.run',
    zip_safe=False,
)



import mock
from django.http import HttpRequest
from django.template import Context, Template
from django.test import TestCase

from metaredirect.helpers import is_interactive_user_agent
from metaredirect.views import redirect_to


class InteractiveUserAgentTestCase(TestCase):
    INTERACTIVE_USER_AGENTS = (
        'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.5; en-GB; rv:1.9.0.10) '
            'Gecko/2009042315 Firefox/3.0.10',
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_6) AppleWebKit/534.24 '
            '(KHTML, like Gecko) Chrome/11.0.696.3 Safari/534.24,gzip(gfe)',
    )
    NONINTERACTIVE_USER_AGENTS = (
        'curl/7.21.4 (universal-apple-darwin11.0) libcurl/7.21.4 '
            'OpenSSL/0.9.8r zlib/1.2.5',
        'Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) '
            'AppleWebKit/534.46 (KHTML, like Gecko) Mobil',  # parses incorrectly
    )

    def get_request_for_user_agent(self, agent):
        request = mock.Mock(spec=HttpRequest)
        request.META = {}
        if agent is not None:
            request.META['HTTP_USER_AGENT'] = agent
        return request

    def test_interactive_user_agents(self):
        for agent in self.INTERACTIVE_USER_AGENTS:
            request = self.get_request_for_user_agent(agent)
            self.assertTrue(is_interactive_user_agent(request))

    def test_noninteractive_user_agents(self):
        for agent in self.NONINTERACTIVE_USER_AGENTS:
            request = self.get_request_for_user_agent(agent)
            self.assertFalse(is_interactive_user_agent(request))

    def test_no_user_agent(self):
        request = self.get_request_for_user_agent(None)
        self.assertFalse(is_interactive_user_agent(request))


class RedirectToViewTestCase(TestCase):
    user_agent_callable = 'metaredirect.views.is_interactive_user_agent'

    def get_response(self, url='http://example.com', *args, **kwargs):
        request = mock.Mock(spec=HttpRequest)
        return redirect_to(request, url, *args, **kwargs)

    @mock.patch(user_agent_callable, return_value=True)
    def test_200_redirect_for_interactive_agent(self, *args):
        url = 'http://example.com'
        response = self.get_response(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['X-Location'], url)

        meta_redirect = \
            '<META http-equiv="refresh" content="0;URL=http://example.com">'
        self.assertContains(response, meta_redirect)

        javascript_redirect = \
            '<script>location.replace("http:\/\/example.com")</script>'
        self.assertContains(response, javascript_redirect)

    @mock.patch(user_agent_callable, return_value=False)
    def test_302_redirect_for_noninteractive_agent(self, *args):
        response = self.get_response()
        self.assertEqual(response.status_code, 302)

    @mock.patch(user_agent_callable, return_value=False)
    def test_301_redirect_for_noninteractive_agent(self, *args):
        response = self.get_response(permanent=True)
        self.assertEqual(response.status_code, 301)

    @mock.patch(user_agent_callable, return_value=True)
    def test_200_redirect_escaping(self, *args):
        response = self.get_response(url='http://example.com/")</script>')
        self.assertEqual(response.status_code, 200)

        meta_redirect = '<META http-equiv="refresh" ' \
            'content="0;URL=http://example.com/&quot;)&lt;/script&gt;">'
        self.assertContains(response, meta_redirect)

        javascript_redirect = r'<script>location.replace' \
            '("http:\/\/example.com\/\u0022)\u003C\/script\u003E")</script>'
        self.assertContains(response, javascript_redirect)


class EscapeForwardSlashesTestCase(TestCase):
    template = Template('{% load escaping %}{{ value|escapeforwardslashes }}')

    def test_escapes_slashes(self):
        self.assertEqual(self.template.render(Context({
            'value': 'foo/bar',
        })), 'foo\/bar')
